@@ -1,5 +1,7 @@
 from .indices import NoopIndex, ColumnsIndex, MultiIndex
-from .similarities import StringSimilarity, DateSimilarity, JaroWinklerSimilarity
+from .similarities import (
+    StringSimilarity, DateSimilarity, JaroWinklerSimilarity, AbsoluteNumericalSimilarity, RelativeNumericalSimilarity
+)
 from .matchers import ThresholdMatcher
 from .variators import Variator, Swap
 from .filters import DissimilarFilter, NonOverlappingFilter
@@ -7,5 +9,6 @@ from .filters import DissimilarFilter, NonOverlappingFilter
 __all__ = [
     "NoopIndex", "ColumnsIndex", "JaroWinklerSimilarity", "StringSimilarity",
     "DateSimilarity", "ThresholdMatcher", "MultiIndex", "Variator", "Swap",
-    "DissimilarFilter", "NonOverlappingFilter"
+    "DissimilarFilter", "NonOverlappingFilter", "AbsoluteNumericalSimilarity",
+    "RelativeNumericalSimilarity",
 ]
