@@ -1,60 +1,67 @@
-import hashlib
+"""
+An index divides the data up into one or more buckets. Only records in the same bucket are matched against each other.
+When used correctly this decreases the number of pairs to compare and speeds up the matching process significantly.
+"""
+
 import operator
 import functools
 import itertools
 from typing import Type
+from abc import ABC, abstractmethod
 
 import pandas as pd
 
 
-class BaseIndex(object):
-    """Base class for all index classes.
+class BaseIndex(ABC):
+    """Abstract base class for all index classes.
 
-    This doesn't have any functionality but simple serve as an interface for all
-    subclasses to implement.
+    Sub-class should implement :meth:`_key_ind_map` method.
+
+    .. automethod:: _key_ind_map
     """
     _dfs: list[tuple[pd.DataFrame, list]]
 
     def __init__(self) -> None:
         self._dfs = list()
 
+    @abstractmethod
     def _key_ind_map(self, df: pd.DataFrame) -> dict:
         """Returns a mapping between bucket keys and row indices.
 
-        This is the mandatory method for all subclassses to implement.
+        :param df: the data to index
+        :type df: :class:`pandas:pandas.DataFrame`
 
-        Args:
-            df (pd.DataFrame): the data to index
-
-        Returns:
-            a mapping between bucket key and all row indices that belong
-            to the bucket. Key could be anything hashable but value must always
-            be a list even if there are only one row.
+        :return: a mapping between bucket key and all row indices that belong
+            to the bucket. Key could be anything hashable but the value must always
+            be a list even if there is only one row.
+        :rtype: :obj:`dict`
         """
         raise NotImplementedError()
 
     def keys(self, df: pd.DataFrame) -> set:
-        """Returns a set of keys that should be used to retrieve buckets
+        """Returns a set of keys that could be used to retrieve buckets
 
-        Args:
-            df (pd.DataFrame): the data to index
+        :param df: the data to index
+        :type df: :class:`pandas:pandas.DataFrame`
 
-        Returns:
-            a set of bucket keys
+        :return: a set of bucket keys
+        :rtype: :obj:`set`
         """
         key_inds = self._key_ind_map(df)
         self._dfs.append((df, key_inds))
         return set(key_inds.keys())
 
     def bucket(self, df: pd.DataFrame, key: any) -> pd.DataFrame:
-        """Retrieve a bucket given the original data and a bucket key
+        """Retrieves a bucket given the original data and a bucket key
 
-        Args:
-            df (pd.DataFrame): the data to index
-            key (any): key of bucket to retrieve
+        :param df: the data to index
+        :type df: :class:`pandas:pandas.DataFrame`
 
-        Returns:
-            rows in bucket
+        :param key: one of the keys returned from :meth:`BaseIndex.keys`
+        :type key: any
+
+        :return: rows in bucket
+        :rtype: :class:`pandas:pandas.DataFrame`
         """
         for frame, key_inds in self._dfs:
             if frame is df:
@@ -72,8 +79,8 @@ class BaseIndex(object):
 class NoopIndex(BaseIndex):
     """Returns all data as a single bucket.
 
-    Using this is like using no index at all. Useful for when there are
-    not too many rows.
+    Using this is like using no index at all. Useful for when you don't care about
+    performance (e.g. when there are not too many rows).
     """
 
     def _key_ind_map(self, df: pd.DataFrame) -> dict:
@@ -85,13 +92,9 @@ class ColumnsIndex(BaseIndex):
     """
 
     def __init__(self, cols: str or list[str]) -> None:
-        """Creates a new instance of ColumnsIndex
-
-        Args:
-            cols (str or list of str): column names to index
-
-        Returns:
-            no value
+        """
+        :param cols: single column name or list of column names to index
+        :type cols: :obj:`str` or :obj:`list` of :obj:`str`
         """
         super().__init__()
         if type(cols) is str:
@@ -115,22 +118,18 @@ class MultiIndex(BaseIndex):
     """Creates bucket keys by combining bucket keys from 2 or more indices.
 
     This has 2 modes of operation:
-    - if `combine_keys` is False then key lists are simply concatenated together.
-    Which mean a more lax pairing outcome.
-    - if `combine_keys` is True then final key lists is the product of all key lists.
-    Thus mean a stricter pairing outcome.
+
+    - When **combine_keys** is `False`: the key sets of each index are concatenated together, this is like OR-ing the keys.
+    - When **combine_keys** is `True`: the final key set is the cartesian product of all key sets, this is like AND-ing the keys.
     """
 
     def __init__(self, indices: list[Type[BaseIndex]], combine_keys: bool = False) -> None:
-        """Creates a new instance of MultiIndex
+        """
+        :param indices: list of indices to combine
+        :type indices: :obj:`list` of :class:`BaseIndex` subclass
 
-        Args:
-            indices (list of index): the indices to combine
-            combine_keys (bool): whether final key list should be the catersian
-                product of all key lists
-
-        Returns:
-            no value
+        :param combine_keys: whether the final key set should be the cartesian product of all key sets, defaults to `False`
+        :type combine_keys: :obj:`bool`
         """
         super().__init__()
         self._indices = indices
